pub struct MapVecMatrix<'a, CF, ColT, RowT>{
pub columns: Cow<'a, FxHashMap<ColT, Vec<(CF, RowT)>>>,
}Expand description
A sprase matrix stored as a FxHashMap<Column> where each Column is a vector of the non-zero entries.
The columns are indexed by any Hash type and the rows are indexed by an arbitrary type.
This is similar to the typical sparse matrix representation in other frameworks.
Fields
columns: Cow<'a, FxHashMap<ColT, Vec<(CF, RowT)>>>The columns of the matrix.
This is stored behind a Cow to allow construction whether you own columns or not